# -*- coding: utf-8 -*-
# Copyright 2020 Paul Bailey
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        https://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from requests_toolbelt.sessions import BaseUrlSession

from mstr.requests.rest import exceptions

MSTR_AUTH_TOKEN = "X-MSTR-AuthToken"
MSTR_PROJECT_ID_HEADER = "X-MSTR-ProjectID"
MSTR_HEADER_PREFIX = "X-MSTR"


class MSTRBaseSession(BaseUrlSession):
    def has_session(self):
        return MSTR_AUTH_TOKEN in self.headers

    def destroy_auth_token(self):
        try:
            del self.headers[MSTR_AUTH_TOKEN]
        except KeyError:
            pass

    def request(self, method, url, include_auth=True, project_id=None, *args, **kwargs):

        headers = kwargs.get("headers", {})

        if include_auth and MSTR_AUTH_TOKEN in self.headers:
            headers.update({MSTR_AUTH_TOKEN: self.headers[MSTR_AUTH_TOKEN]})

        if project_id is not None:
            headers.update({MSTR_PROJECT_ID_HEADER: project_id})

        kwargs["headers"] = headers

        response = super(MSTRBaseSession, self).request(method, url, *args, **kwargs)

        if not response.ok and response.headers["content-type"] == "application/json":
            try:
                resp_json = response.json()
                try:
                    resp_code = resp_json["code"]
                except KeyError:
                    raise exceptions.MSTRUnknownException(**resp_json)
                if resp_code in ["ERR003", "ERR001"]:
                    raise exceptions.LoginFailureException(**resp_json)
                elif resp_code == "ERR009":
                    raise exceptions.SessionException(**resp_json)
                elif resp_code == "ERR004":
                    raise exceptions.ResourceNotFoundException(**resp_json)
                else:
                    raise exceptions.MSTRException(**resp_json)
            except ValueError:
                raise exceptions.MSTRException(
                    "Couldn't parse response: {}".format(response.text)
                )
        if response.ok:
            for key, value in response.headers.items():
                if key.upper().startswith("X-MSTR"):
                    self.headers.update({key: value})
        return response



# -*- coding: utf-8 -*-
# Copyright 2020 Paul Bailey
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        https://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.


class AuthMixin:
    def post_login(self, username=None, password=None, application_type: int = 8):
        """``POST``s a login request."""
        if username is not None and password is not None:
            data = {
                "username": username,
                "password": password,
                "loginMode": 1,
                "applicationType": application_type,
            }
        else:
            data = {
                "loginMode": 8,
                "applicationType": application_type,
            }
        login_response = self.post("auth/login", json=data)
        if login_response.status_code != 204:
            login_response.raise_for_status()
        return login_response

    def post_logout(self):
        logout_response = self.post("auth/logout")
        if logout_response.status_code == 204:
            self.destroy_auth_token()

    # "Friendly" method aliases
    def login(
        self, username: str = None, password: str = None, application_type: int = 8
    ):
        """Logs in to MicroStrategy REST API.

        These credentials must be using MicroStrategy's standard authentication.

        If no credentials are provided, the session will attempt to establish an anonymous connection.

        Returns:
            A ``requests`` response object with result of the login request
        """
        return self.post_login(username, password, application_type)

    def logout(self):
        """Closes the REST API session associated with the session object."""
        return self.post_logout()



# -*- coding: utf-8 -*-
# Copyright 2020 Paul Bailey
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        https://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from .utils import check_valid_session


class DatasetStatus:
    UNKNOWN = -1
    READY = 1
    PROCESSING = 2
    NOT_READY = 4

    @staticmethod
    def is_ready(status):
        return status & DatasetStatus.READY == DatasetStatus.READY

    @staticmethod
    def is_processing(status):
        return status & DatasetStatus.PROCESSING == DatasetStatus.PROCESSING

    @staticmethod
    def is_not_ready(status):
        return status & DatasetStatus.NOT_READY == DatasetStatus.NOT_READY


class DatasetsMixin:
    @check_valid_session
    def get_datasets_instance_status(self, project_id, dataset_id, instance_id):
        return self.get(
            "datasets/{}/instances/{}/status".format(dataset_id, instance_id),
            project_id=project_id,
        )

    # "Friendly" method aliases
    def get_dataset_status(self, project_id, dataset_id, instance_id):
        return DatasetStatusResponse(
            self.get_datasets_instance_status(project_id, dataset_id, instance_id)
        )


class DatasetStatusResponse:
    def __init__(self, response):
        if response.headers["content-type"] == "application/json":
            json = response.json()
            self.code = json.get("code", -1)
            self.message = json.get("message", "")
        else:
            self.code = -1
            self.message = "No error message found"



# -*- coding: utf-8 -*-
# Copyright 2020 Paul Bailey
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        https://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from .base import MSTRBaseSession
from .exceptions import (
    ExecutionCancelledException,
    ExecutionFailedException,
    MSTRUnknownException,
)
from .execution_status import CUBE_RUNNING, CUBE_PUBLISHED, CUBE_PREPARING
from .mixins import SessionPersistenceMixin
from mstr.requests.rest.api import (
    AuthMixin,
    SessionsMixin,
    ObjectsMixin,
    DatasetsMixin,
    CubesMixin,
)


class MSTRRESTSession(
    ObjectsMixin,
    DatasetsMixin,
    CubesMixin,
    AuthMixin,
    SessionsMixin,
    SessionPersistenceMixin,
    MSTRBaseSession,
):
    pass

    def execute_dataset_object(self, project_id, object_id):
        response = self.post(
            "cubes/{}".format(object_id), headers={"X-MSTR-ProjectID": project_id}
        )
        if response.ok:
            return response.json()["instanceId"]

    def get_dataset_instance_status(self, project_id, object_id, instance_id):
        response = self.get(
            "datasets/{}/instances/{}/status".format(object_id, instance_id),
            headers={"X-MSTR-ProjectID": project_id},
        )
        if response.ok:
            resp_json = response.json()
            status = resp_json.get("code", None)
            if status:
                if status & CUBE_PUBLISHED == CUBE_PUBLISHED:
                    return CUBE_PUBLISHED
                elif status & CUBE_RUNNING == CUBE_RUNNING:
                    return CUBE_RUNNING
                elif status & CUBE_PREPARING == CUBE_PREPARING:
                    return CUBE_PREPARING
                else:
                    raise MSTRUnknownException(**resp_json)
            else:
                if "User request is cancelled" in resp_json.get("message", ""):
                    raise ExecutionCancelledException(**resp_json)
                else:
                    raise ExecutionFailedException(**resp_json)



# -*- coding: utf-8 -*-
# Copyright 2020 Paul Bailey
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        https://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from .utils import check_valid_session


class SessionsMixin:
    @check_valid_session
    def put_sessions(self):
        return self.put("sessions")

    @check_valid_session
    def get_sessions_userinfo(self):
        return self.get("sessions/userInfo")

    @check_valid_session
    def get_sessions(self):
        return self.get("sessions")

    # "Friendly" method aliases
    def extend_session(self):
        return self.put_sessions()

    def get_userinfo(self):
        return self.get_sessions_userinfo()

    def get_session_info(self):
        return self.get_sessions()



# -*- coding: utf-8 -*-
# Copyright 2020 Paul Bailey
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        https://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from .utils import check_valid_session


class ObjectTypes:
    REPORT = 3
    FOLDER = 8


class Object:
    def __init__(self, response):
        obj_json = response.json()

        ancestors = obj_json.get("ancestors", [])
        self.name = obj_json.get("name", "NAME N/A")

        path_list = sorted(ancestors, key=lambda x: x["level"], reverse=True)
        self.path = "/".join(list(map(lambda x: x["name"], path_list)))

    def full_name(self):
        return "{}/{}".format(self.path, self.name)


class ObjectsMixin:
    @check_valid_session
    def get_objects(self, project_id, object_id, object_type=ObjectTypes.REPORT):
        return self.get(
            "objects/{}?type={}".format(object_id, object_type), project_id=project_id
        )

    # "Friendly" method aliases
    def get_object_metadata(self, project_id, object_id, object_type):
        return self.get_object_metadata(project_id, object_id, object_type)

    def get_object_details(self, project_id, object_id, object_type):
        obj_json = self.get_objects(project_id, object_id, object_type).json()

        ancestors = obj_json.get("ancestors", [])
        name = obj_json.get("name", "NAME N/A")

        path_list = sorted(ancestors, key=lambda x: x["level"], reverse=True)
        path = "/".join(list(map(lambda x: x["name"], path_list)))

        return {"name": name, "path": path, "full": "{}/{}".format(path, name)}



# -*- coding: utf-8 -*-
# Copyright 2020 Paul Bailey
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        https://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from .utils import check_valid_session


class CubesMixin:
    @check_valid_session
    def post_cubes(self, project_id, cube_id):
        return self.post_cubes("cubes/{}".format(cube_id), project_id=project_id)

    # "Friendly" method aliases

    def publish_cube(self, project_id, cube_id):
        return self.post_cubes(project_id, cube_id)



# -*- coding: utf-8 -*-
# Copyright 2020 Paul Bailey
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        https://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from .errors import iserver_error_codes


class MSTRException(Exception):
    def __init__(self, message=None, *args, **kwargs):
        self.code = kwargs.get("code", "N/A")
        self.message = "{}: {}.".format(self.code, message)
        self.iserver_code = kwargs.get("iServerCode", None)
        if self.iserver_code:
            self.iserver_message = iserver_error_codes.get(self.iserver_code)
            self.message += "  [I-Server error code {} ({})]".format(
                self.iserver_message, self.iserver_code
            )
        super().__init__(self.message, *args)


class LoginFailureException(MSTRException):
    pass


class SessionException(MSTRException):
    pass


class ExecutionCancelledException(MSTRException):
    pass


class MSTRUnknownException(MSTRException):
    pass


class ExecutionFailedException(MSTRException):
    pass


class ResourceNotFoundException(MSTRException):
    pass



# -*- coding: utf-8 -*-
# Copyright 2020 Paul Bailey
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        https://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from .exceptions import SessionException

import json

from requests.utils import dict_from_cookiejar, cookiejar_from_dict


class SessionPersistenceMixin:
    def json(self):
        return json.dumps(
            {
                "base_url": self.base_url,
                "cookies": dict_from_cookiejar(self.cookies),
                "headers": self.headers,
            }
        )

    def update_from_json(self, data):
        if type(data) is dict:
            input_data = data
        elif type(data) is str:
            input_data = json.loads(data)
        else:
            input_data = dict()

        try:
            self.base_url = input_data["base_url"]
            self.cookies = cookiejar_from_dict(input_data["cookies"])
            self.headers.update(input_data["headers"])
        except KeyError as e:
            raise SessionException(e)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mstr', 'mstr.requests', 'mstr.requests.rest', 'mstr.requests.rest.api']

package_data = \
{'': ['*']}

install_requires = \
['requests', 'requests-toolbelt']

setup_kwargs = {
    'name': 'mstr-rest-requests',
    'version': '0.12.2',
    'description': 'Easily make requests of the MicroStrategy REST API',
    'long_description': "# mstr-rest-requests\n\nA extension to the excellent `requests` `Session` object, to enable more straightforward interaction with MicroStrategy's REST API.\n\n![Python package](https://github.com/paulbailey/mstr-rest-requests/workflows/Python%20package/badge.svg)\n\n## Usage\n\n### Installation\n\nSimply install the package however you normally install them, for example:\n\n`pip install mstr-rest-requests`\n\n### Examples\n\n#### Authentication\n\nHere's how to get an authenticated session (currently only standard and anonymous authentication are supported):\n\n```\nfrom mstr.requests import MSTRRESTSession\n\nsession = MSTRRESTSession(base_url='https://demo.microstrategy.com/MicroStrategyLibrary/api/')\nsession.login(username='dave', password='hellodave')\nsession.has_session()\n# returns True\n```\n\n#### Session handling\n\nSeveral convenience methods are implemented to make dealing with Session objects easier.\n\n`def has_session(self)`\n\nWill return a boolean as to whether the session contains an authentication tokem.\n\n`def destroy_auth_token(self)`\n\nRemoves the auth token from the session\n\n`def json(self)`\n\nReturns a JSON representation of the session that can be reconstituted with:\n\n`update_from_json(self, data)`\n\nwhere `data` is either a dict or a string containing JSON data.\n\n#### HTTP requests\n\nThe MSTRRESTSession adds two parameters to all request methods:\n\n`include_auth=True, project_id=None`\n\nso you can specify a `project_id` on any request by adding the parameter.\n\n#### Convenience methods for API calls\n\nTODO\n",
    'author': 'Paul Bailey',
    'author_email': 'bailey@dreamshake.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/paulbailey/mstr-rest-requests',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.5',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
# Copyright 2020 Paul Bailey
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        https://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from .session import MSTRRESTSession


class AuthenticatedMSTRRESTSession(MSTRRESTSession):
    """A context manager for sessions interacting with the MicroStrategy REST API."""

    def __init__(self, base_url, username=None, password=None):
        super(AuthenticatedMSTRRESTSession, self).__init__(base_url)
        self._username = username
        self._password = password

    def __enter__(self):
        self.login(self._username, self._password)
        return self

    def __exit__(self, t, v, tb):
        self.logout()
